@@ -48,7 +48,7 @@ from . import debug
 class AXObject:
     """Utilities for obtaining information about accessible objects."""
 
-    KNOWN_DEAD = []
+    KNOWN_DEAD = {}
     REAL_APP_FOR_MUTTER_FRAME = {}
     REAL_FRAME_FOR_MUTTER_FRAME = {}
 
@@ -61,22 +61,19 @@ class AXObject:
         while True:
             time.sleep(60)
             with AXObject._lock:
-                tokens = ["AXObject: Clearing", len(AXObject.KNOWN_DEAD), "known-dead objects"]
+                tokens = ["AXObject: Clearing known dead-or-alive state for",
+                          len(AXObject.KNOWN_DEAD), "objects"]
                 debug.printTokens(debug.LEVEL_INFO, tokens, True)
                 AXObject.KNOWN_DEAD.clear()
 
-                msg = (
-                    f"AXObject: Clearing {len(AXObject.REAL_APP_FOR_MUTTER_FRAME)} "
-                    f"real app for mutter frame"
-                )
-                debug.println(debug.LEVEL_INFO, msg, True)
+                tokens = ["AXObject: Clearing", len(AXObject.REAL_APP_FOR_MUTTER_FRAME),
+                          "real apps for mutter frames"]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
                 AXObject.REAL_APP_FOR_MUTTER_FRAME.clear()
 
-                msg = (
-                    f"AXObject: Clearing {len(AXObject.REAL_FRAME_FOR_MUTTER_FRAME)} "
-                    "real frame for mutter frame"
-                )
-                debug.println(debug.LEVEL_INFO, msg, True)
+                tokens = ["AXObject: Clearing", len(AXObject.REAL_FRAME_FOR_MUTTER_FRAME),
+                          "real frames for mutter frames"]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
                 AXObject.REAL_FRAME_FOR_MUTTER_FRAME.clear()
 
 
@@ -98,7 +95,28 @@ class AXObject:
     def object_is_known_dead(obj):
         """Returns True if we know for certain this object no longer exists"""
 
-        return hash(obj) in AXObject.KNOWN_DEAD
+        return obj and AXObject.KNOWN_DEAD.get(hash(obj)) is True
+
+    @staticmethod
+    def _set_known_dead_status(obj, is_dead):
+        """Updates the known-dead status of obj"""
+
+        if obj is None:
+            return
+
+        current_status = AXObject.KNOWN_DEAD.get(hash(obj))
+        if current_status == is_dead:
+            return
+
+        AXObject.KNOWN_DEAD[hash(obj)] = is_dead
+        if is_dead:
+            msg = "AXObject: Adding to known dead objects"
+            debug.printMessage(debug.LEVEL_INFO, msg, True, True)
+            return
+
+        if current_status:
+            tokens = ["AXObject: Removing", obj, "from known-dead objects"]
+            debug.printTokens(debug.LEVEL_INFO, msg, tokens, True)
 
     @staticmethod
     def handle_error(obj, error, msg):
@@ -106,13 +124,17 @@ class AXObject:
 
         error = str(error)
         if re.search(r"accessible/\d+ does not exist", error):
-            AXObject.KNOWN_DEAD.append(hash(obj))
             msg = msg.replace(error, "object no longer exists")
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
         elif re.search(r"The application no longer exists", error):
-            AXObject.KNOWN_DEAD.append(hash(obj))
             msg = msg.replace(error, "app no longer exists")
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+        else:
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            return
 
-        debug.println(debug.LEVEL_INFO, msg, True)
+        if AXObject.KNOWN_DEAD.get(hash(obj)) is False:
+            AXObject._set_known_dead_status(obj, True)
 
     @staticmethod
     def supports_action(obj):
@@ -457,11 +479,9 @@ class AXObject:
         parent = AXObject.get_parent_checked(obj)
         while parent:
             if parent in objects:
-                msg = (
-                    f"AXObject: Circular tree suspected in find_ancestor. "
-                    f"{parent} already in: {' '.join(map(str, objects))}"
-                )
-                debug.println(debug.LEVEL_INFO, msg, True)
+                tokens = ["AXObject: Circular tree suspected in find_ancestor. ",
+                          parent, "already in: ", objects]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
                 return None
 
             if pred(parent):
@@ -549,11 +569,9 @@ class AXObject:
             return reported_child
 
         if real_child != reported_child:
-            msg = (
-                f"AXObject: {container}'s child at {index} is {real_child}; "
-                f"not reported child {reported_child}. "
-            )
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["AXObject: ", container, f"'s child at {index} is ", real_child,
+                      "; not reported child", reported_child]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         return real_child
 
@@ -581,8 +599,8 @@ class AXObject:
 
         start = time.time()
         result = AXObject._find_descendant(obj, pred)
-        msg = f"AXObject: find_descendant: found {result} in {time.time() - start:.4f}s"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["AXObject: find_descendant: found", result, f"in {time.time() - start:.4f}s"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return result
 
     @staticmethod
@@ -625,7 +643,7 @@ class AXObject:
             f"AXObject: find_all_descendants: {len(matches)} "
             f"matches found in {time.time() - start:.4f}s"
         )
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return matches
 
     @staticmethod
@@ -642,6 +660,7 @@ class AXObject:
             AXObject.handle_error(obj, error, msg)
             return Atspi.Role.INVALID
 
+        AXObject._set_known_dead_status(obj, False)
         return role
 
     @staticmethod
@@ -674,6 +693,7 @@ class AXObject:
             AXObject.handle_error(obj, error, msg)
             return ""
 
+        AXObject._set_known_dead_status(obj, False)
         return name
 
     @staticmethod
@@ -842,6 +862,7 @@ class AXObject:
             AXObject.handle_error(obj, error, msg)
             return Atspi.StateSet()
 
+        AXObject._set_known_dead_status(obj, False)
         return state_set
 
     @staticmethod
@@ -924,8 +945,8 @@ class AXObject:
         # We want to avoid self-referential relationships.
         type_includes_object = [Atspi.RelationType.MEMBER_OF]
         if relation_type not in type_includes_object and obj in targets:
-            msg = f'ERROR: {obj} is in its own {relation_type} target list'
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["AXObject: ", obj, "is in its own", relation_type, "target list"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             targets.remove(obj)
 
         return list(targets)
@@ -1145,6 +1166,7 @@ class AXObject:
             AXObject.handle_error(obj, error, msg)
             return True
 
+        AXObject._set_known_dead_status(obj, False)
         return False
 
     @staticmethod
