@@ -26,11 +26,16 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2010 Joanmarie Diggs."
 __license__   = "LGPL"
 
-import pyatspi
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
 
+import orca.debug as debug
 import orca.messages as messages
 import orca.scripts.toolkits.GAIL as GAIL
-import orca.speech as speech
+import orca.settings as settings
+from orca.ax_object import AXObject
+from orca.ax_utilities import AXUtilities
 
 from .chat import Chat
 from .script_utilities import Utilities
@@ -53,13 +58,13 @@ class Script(GAIL.Script):
 
         # So we can take an educated guess at identifying the buddy list.
         #
-        self._buddyListAncestries = [[pyatspi.ROLE_TREE_TABLE,
-                                      pyatspi.ROLE_SCROLL_PANE,
-                                      pyatspi.ROLE_FILLER,
-                                      pyatspi.ROLE_PAGE_TAB,
-                                      pyatspi.ROLE_PAGE_TAB_LIST,
-                                      pyatspi.ROLE_FILLER,
-                                      pyatspi.ROLE_FRAME]]
+        self._buddyListAncestries = [[Atspi.Role.TREE_TABLE,
+                                      Atspi.Role.SCROLL_PANE,
+                                      Atspi.Role.FILLER,
+                                      Atspi.Role.PAGE_TAB,
+                                      Atspi.Role.PAGE_TAB_LIST,
+                                      Atspi.Role.FILLER,
+                                      Atspi.Role.FRAME]]
 
         GAIL.Script.__init__(self, app)
 
@@ -74,7 +79,7 @@ class Script(GAIL.Script):
         return SpeechGenerator(self)
 
     def getUtilities(self):
-        """Returns the utilites for this script."""
+        """Returns the utilities for this script."""
 
         return Utilities(self)
 
@@ -104,20 +109,16 @@ class Script(GAIL.Script):
 
         return self.chat.getPreferencesFromGUI()
 
-    def onChildrenChanged(self, event):
-        """Called whenever a child object changes in some way.
-
-        Arguments:
-        - event: the text inserted Event
-        """
+    def onChildrenAdded(self, event):
+        """Callback for object:children-changed:add accessibility events."""
 
         # Check to see if a new chat room tab has been created and if it
         # has, then announce its name. See bug #469098 for more details.
         #
         if event.type.startswith("object:children-changed:add"):
-            rolesList = [pyatspi.ROLE_PAGE_TAB_LIST,
-                         pyatspi.ROLE_FILLER,
-                         pyatspi.ROLE_FRAME]
+            rolesList = [Atspi.Role.PAGE_TAB_LIST,
+                         Atspi.Role.FILLER,
+                         Atspi.Role.FRAME]
             if self.utilities.hasMatchingHierarchy(event.source, rolesList):
                 # As it's possible to get this component hierarchy in other
                 # places than the chat room (i.e. the Preferences dialog),
@@ -127,15 +128,22 @@ class Script(GAIL.Script):
                 # last child has a name.
                 #
                 nameFound = False
-                frameName = event.source.parent.parent.name
-                for child in event.source:
-                    if frameName and (frameName == child.name):
+                frame = AXObject.find_ancestor(event.source,
+                                               lambda x: AXObject.get_role(x) == Atspi.Role.FRAME)
+                frameName = AXObject.get_name(frame)
+                if not frameName:
+                    return
+                for child in AXObject.iter_children(event.source):
+                    if frameName == AXObject.get_name(child):
                         nameFound = True
+                        break
                 if nameFound:
-                    child = event.source[-1]
-                    if child.name:
-                        line = messages.CHAT_NEW_TAB % child.name
-                        speech.speak(line)
+                    child = AXObject.get_child(event.source, -1)
+                    childName = AXObject.get_name(child)
+                    if childName:
+                        line = messages.CHAT_NEW_TAB % childName
+                        voice = self.speechGenerator.voice(obj=child, string=line)
+                        self.speakMessage(line, voice=voice)
 
     def onNameChanged(self, event):
         """Called whenever a property on an object changes.
@@ -185,11 +193,22 @@ class Script(GAIL.Script):
     def onWindowActivated(self, event):
         """Called whenever a toplevel window is activated."""
 
+        if not settings.enableSadPidginHack:
+            msg = "PIDGIN: Hack for missing events disabled"
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            GAIL.Script.onWindowActivated(self, event)
+            return
+
+        msg = "PIDGIN: Starting hack for missing events"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
         # Hack to "tickle" the accessible hierarchy. Otherwise, the
         # events we need to present text added to the chatroom are
         # missing.
-        hasRole = lambda x: x and x.getRole() == pyatspi.ROLE_PAGE_TAB
-        allPageTabs = pyatspi.findAllDescendants(event.source, hasRole)
+        AXUtilities.find_all_page_tabs(event.source)
+
+        msg = "PIDGIN: Hack to work around missing events complete"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         GAIL.Script.onWindowActivated(self, event)
 
     def onExpandedChanged(self, event):
@@ -198,9 +217,8 @@ class Script(GAIL.Script):
         # Overridden here because the event.source is in a hidden column.
         obj = event.source
         if self.chat.isInBuddyList(obj):
-            obj = obj.parent[obj.getIndexInParent() + 1]
-            self.updateBraille(obj)
-            speech.speak(self.speechGenerator.generateSpeech(obj, alreadyFocused=True))
+            obj = AXObject.get_next_sibling(obj)
+            self.presentObject(obj, alreadyFocused=True)
             return
-            
+
         GAIL.Script.onExpandedChanged(self, event)
